/*
* 32bit vector & scolar addition implementation.
*
* function prototype
* void fr32_vector_add(
*        const fract32 a,
*        const fract32 b[],
*        fract32 c[],
*        int count);
*
* parameters
*   FP+20	-	int count
*   FP+16	R2	const fr32 c[] : as result
*	FP+12	R1	const fr32 b[]
*	FP+ 8	R0	const fr32 a
*
* return
*	none
*
* side effect
*   out[] : obtain output data
*

* register layout
*   P2 : out
*	P3 : count : loop counter's initial value
*   I0 : none
*	I1 : a[]
*   I2 : c[]
*   R0 : a
*   R1 : b[i]
*	R3 : a[i]+b
*/

	.text
	.align 4
	.global _fr32_vector_add_svv;
	.type _fr32_vector_add_svv, STT_FUNC;

_fr32_vector_add_svv:
	link	0;
	[--sp] = (r7:4, p5:3);		// save all preserved register

		/* Set up registers */
	i1 = r1;
	i2 = r2;
	p3 = [fp+20];				// load count
	p3 += -2;

		/* outer loop */
		r1 = [i1++];
		r3 = r0 + r1(s) || r1 = [i1++];
	loop count lc0 = p3;	// Todo : can be 3 parallel instruction. But seems to be simulator bug in 2013RC1RC
	loop_begin count;
	 	[i2++] = r3;
		r3 = r1 + r0(s) ||	r1 = [i1++] ;
	loop_end count;
		[i2++] = r3;
		r3 = r0 + r1(s);
		[i2++] = r3;
		/* end of outer loop */


	(r7:4, p5:3) = [sp++];		// restore all preserved register
	unlink;
	rts;
	.size	_fr32_vector_add_svv, .-_fr32_vector_add_svv
