/*
* 32bit complex vector * complex multiplying implementation. Round at 1bit under LSB twice.
*
* function prototype
* void fr32_vector_complex_mul(
*        const fract32 ar[],
*        const fract32 ai[],
*        const fract32 br[],
*        const fract32 bi[],
*        fract32 cr[],
*        fract32 ci[],
*        int count
*
* parameters
*   FP+28	-	int count
*   FP+28	-	fr32 ci[]
*   FP+24	-	fr32 cr[]
*   FP+20	-	const fr32 bi[]
*   FP+16	R2	const fr32 br[]
*	FP+12	R1	const fr32 ai[]
*	FP+ 8	R0	const fr32 ar[]
*
* return
*	none
*
* side effect
*   cr[], c[i] : obtain output data.
*

* register layout
*   I0 : ar[]
*   I1 : ai[]
*   I2 : br[]
*   I2 : bi[]
*   P0 : cr[]
*   P1 : ci[]
*	P3 : count : loop counter's initial value
*   R0 : ar[i]
*   R1 : ai[i]
*	R2 : br[i]
*	R3 : bi[i]
*	R6 : a[i]*b[i]
*	R7 : rounding constant.
*/



	.text
	.align 4
	.global _fr32_vector_complex_mul;
	.type _fr32_vector_complex_mul, STT_FUNC;

_fr32_vector_complex_mul:
	link	0;
	[--sp] = (r7:4, p5:3);		// save all preserved register

		/* Set up registers */
	i0 = r0;			// ar
	i1 = r1;			// ai
	i2 = r2;			// br
	r3 = [fp+20];		// bi
	i3 = r3;
	p0 = [fp+24];		// cr
	p1 = [fp+28];		// ci
	p3 = [fp+32];		// load count

	r5.H = 0xFFF4;		// (bit14 for rounding - offset of L*L) << 4
	r7.H = 0x0800;		// >> 4

	r5.L = 0x8000;		// -1.0
	r7.L = 0x8000;		// -1.0

		/* outer loop */
	loop count lc0 = p3;
	loop_begin count;

				// r7 is bias of rounding point
				/* real part */
		//	We make 2.0 by signed int. This 1.0 by unsinged int, which is actually we need.
		a0 = r5.L * r7.L || r0 = [i0++] || r2 = [i2++];				// load coeff & delaydata
		a0 += r5.L * r7.L, a1 = r5.H * r7.H || r1 = [i1++] || r3 = [i3++];

		a1 += r0.H * r2.L (m), a0 += r0.L * r2.L (fu);		// r * r
		a1 -= r1.H * r3.L (m), a0 -= r1.L * r3.L (fu);		// i * i

				/* Second half */
		a0 = a0 >> 16;					// scale down the unsigned integer.
		a0 += a1;
		a1 = a0;

		a0  = r2.H * r0.H, a1 += r2.H * r0.L (m);
		a0 -= r3.H * r1.H, a1 -= r3.H * r1.L (m);

		a1 = a1 >>> 15;				// scale down the signed integer
		r6 = (a0 += a1);


				// r7 is bias of rounding point
				/* imagenary part */
		a0 = r5.L * r7.L || 		[p0++] = r6;				// store output sample,  a0 = +1.0
		a0 += r5.L * r7.L, 	a1 = r5.H * r7.H;	// a1 = 0xFF_FFFF_4000;

		a1 += r0.H * r3.L (m), a0 += r0.L * r3.L (fu);		// r * r
		a1 += r1.H * r2.L (m), a0 += r1.L * r2.L (fu);		// i * i

				/* Second half */
		a0 = a0 >> 16;					// scale down the unsigned integer.
		a0 += a1;
		a1 = a0;

		a0  = r3.H * r0.H, a1 += r3.H * r0.L (m);
		a0 += r2.H * r1.H, a1 += r2.H * r1.L (m);

		a1 = a1 >>> 15;				// scale down the signed integer
		r6 = (a0 += a1);
		[p1++] = r6;				// store output sample


	loop_end count;
		/* end of outer loop */


	(r7:4, p5:3) = [sp++];		// restore all preserved register
	unlink;
	rts;
	.size	_fr32_vector_complex_mul, .-_fr32_vector_complex_mul
