/*
* 32bit vector Subtraction implementation.
*
* function prototype
* void fr32_vector_add(
*        const fract32 a[],
*        const fract32 b[],
*        fract32 c[],	// a - b -> c
*        int count);
*
* parameters
*   FP+20	-	int count
*   FP+16	R2	const fr32 c[] : as result
*	FP+12	R1	const fr32 b[]
*	FP+ 8	R0	const fr32 a[]
*
* return
*	none
*
* side effect
*   out[] : obtain output data.
*

* register layout
*   P2 : out
*	P3 : count : loop counter's initial value
*   I0 : a[]
*   I1 : b[]
*   I2 : c[]
*   R0 : a[i]
*   R1 : b[i]
*	R3 : a[i]+b[i]
*/

	.text
	.align 4
	.global _fr32_vector_sub;
	.type _fr32_vector_sub, STT_FUNC;

_fr32_vector_sub:
	link	0;
	[--sp] = (r7:4, p5:3);		// save all preserved register

		/* Set up registers */
	i0 = r0;
	i1 = r1;
	i2 = r2;
	p3 = [fp+20];				// load count
	p3 += -1;

		/* outer loop */
		r0 = [i0++];r1 = [i1++];
	loop count lc0 = p3;
	loop_begin count;
		r3 = r0 - r1(s) || r0 = [i0++] || r1 = [i1++];
		[i2++] = r3;
	loop_end count;
		r3 = r0 - r1(s);
		[i2++] = r3;
		/* end of outer loop */


	(r7:4, p5:3) = [sp++];		// restore all preserved register
	unlink;
	rts;
	.size	_fr32_vector_sub, .-_fr32_vector_sub
