/*
 * fx_vector_test.c
 *
 *  Created on: 2014/01/19
 *      Author: takemasa
 */

#include "fx_vector_test.h"
#include <stdio.h>


#define SATP(x)  (((x)>2147483647LL)?(2147483647LL):x)
#define SAT(x) (((x)>=-2147483648LL)?(SATP(x)):-2147483648LL)

#define mult_fr1x32x32(x,y) SAT(((long long )(x)*(long long)(y)+0x40000000)>>31)
#define neg_mult_fr1x32x32(x,y) SAT((-(long long )(x)*(long long)(y)+0x40000000)>>31)


void clearBuffer ( fract32 buf[], int count)
{
    int i;
    for ( i= 0; i<count; i++ )
        buf[i] = 0;
}

/*
 * Basic test to see vector + vector addition.
 */
#define NUMSAMPLE_01 4

fract32 buf_a_01[NUMSAMPLE_01]=
    {
        0x08000000,     //
        0x7FFFFFFF,     //
        0x01000000,
        0x20000000
    };

fract32 buf_b_01[NUMSAMPLE_01] =
    {
        0x01000000,         //
        0x00000001,         //
        0x02000000,         //
        0x00000000
    };

fract32 desired_01[NUMSAMPLE_01] =
    {
        0x09000000,         // Simple addition
        0x7FFFFFFF,         // Saturated
        0x03000000,         // dummy
        0x00000000,         // 0 for count test
    };

void test_01_fr32_vector_add()
{
    fract32 output[NUMSAMPLE_01];
    int i;


        // clear output buffer
    clearBuffer( output, NUMSAMPLE_01);
        // test addition. Sample is less than NUMSAMPLE_01 to test the count parameter
    fr32_vector_add( buf_a_01, buf_b_01, output, NUMSAMPLE_01-1);

    for ( i=0; i<NUMSAMPLE_01; i++)
    {
        if ( output[i] != desired_01[i] )
        {
            printf( "test_01 NG :output[%2d] = 0x%08X but should be 0x%08X\n", i, output[i], desired_01[i] );
            return;
        }
    }
    printf ("test_01 OK\n");
}

#undef TAPS_01
#undef NUMSAMPLE_01

/*
 * Basic test to see scalar + vector addition.
 */
#define NUMSAMPLE_02 4

fract32 buf_a_02[NUMSAMPLE_02]=
    {
        0x08000000,     //
        0x7FFFFFFF,     //
        0x02000000,
        0x20000000
    };


fract32 desired_02[NUMSAMPLE_02] =
    {
        0x09000000,         // Simple addition
        0x7FFFFFFF,         // Saturated
        0x03000000,         // dummy
        0x00000000,         // 0 for count test
    };

void test_02_fr32_vector_add_svv()
{
    fract32 output[NUMSAMPLE_02];
    int i;


        // clear output buffer
    clearBuffer( output, NUMSAMPLE_02);
        // test addition. Sample is less than NUMSAMPLE_02 to test the count parameter
    fr32_vector_add_svv( 0x01000000, buf_a_02, output, NUMSAMPLE_02-1);

    for ( i=0; i<NUMSAMPLE_02; i++)
    {
        if ( output[i] != desired_02[i] )
        {
            printf( "test_02 NG :output[%2d] = 0x%08X but should be 0x%08X\n", i, output[i], desired_02[i] );
            return;
        }
    }
    printf ("test_02 OK\n");
}

#undef TAPS_02
#undef NUMSAMPLE_02

/*
 * Basic test to see vector + vector addition.
 */
#define NUMSAMPLE_03 4

fract32 buf_a_03[NUMSAMPLE_03]=
    {
        0x09000000,     //
        0x80000000,     //
        0x09000000,
        0x20000000
    };

fract32 buf_b_03[NUMSAMPLE_03] =
    {
        0x01000000,         //
        0x00000001,         //
        0x08000000,         //
        0x00000000
    };

fract32 desired_03[NUMSAMPLE_03] =
    {
        0x08000000,         // Simple addition
        0x80000000,         // Saturated
        0x01000000,         // dummy
        0x00000000,         // 0 for count test
    };

void test_03_fr32_vector_sub()
{
    fract32 output[NUMSAMPLE_03];
    int i;


        // clear output buffer
    clearBuffer( output, NUMSAMPLE_03);
        // test subtraction. Sample is less than NUMSAMPLE_03 to test the count parameter
    fr32_vector_sub( buf_a_03, buf_b_03, output, NUMSAMPLE_03-1);

    for ( i=0; i<NUMSAMPLE_03; i++)
    {
        if ( output[i] != desired_03[i] )
        {
            printf( "test_03 NG :output[%2d] = 0x%08X but should be 0x%08X\n", i, output[i], desired_03[i] );
            return;
        }
    }
    printf ("test_03 OK\n");
}

#undef TAPS_03
#undef NUMSAMPLE_03

/*
 * Basic test to see scalar + vector addition.
 */
#define NUMSAMPLE_04 4

fract32 buf_a_04[NUMSAMPLE_04]=
    {
        0x01000000,     //
        0x80000000,     //
        0x08000000,
        0x20000000
    };


fract32 desired_04[NUMSAMPLE_04] =
    {
        0x08000000,         // Simple addition
        0x7fffffff,         // Saturated
        0x01000000,         // dummy
        0x00000000,         // 0 for count test
    };

void test_04_fr32_vector_sub_svv()
{
    fract32 output[NUMSAMPLE_04];
    int i;


        // clear output buffer
    clearBuffer( output, NUMSAMPLE_04);
        // test subtraction. Sample is less than NUMSAMPLE_04 to test the count parameter
    fr32_vector_sub_svv( 0x09000000, buf_a_04, output, NUMSAMPLE_04-1);

    for ( i=0; i<NUMSAMPLE_04; i++)
    {
        if ( output[i] != desired_04[i] )
        {
            printf( "test_04 NG :output[%2d] = 0x%08X but should be 0x%08X\n", i, output[i], desired_04[i] );
            return;
        }
    }
    printf ("test_04 OK\n");
}

#undef TAPS_04
#undef NUMSAMPLE_04


/*
 * Basic test to see scalar + vector addition.
 */
#define NUMSAMPLE_05 15

fract32 buf_a_05[NUMSAMPLE_05]=
    {
        0x40008000,     // summation test for 4 partial products
        0x80000000,     // -1*H,
        0x80000000,     // -1*L
        0x10000000,     // H*-1
        0x00001000,     // L*-1
        0x00008000,     // L*L test 1
        0x00008000,     // L*L test 2
        0x00008000,     // L*L test 3
        0x00000002,     // H*L test 1
        0x00000002,     // H*L test 2
        0x00000002,     // H*L test 3
        0x40000000,
        0x20000000,
        0x10000000,
        0x11110000,     // dummy
    };


fract32 buf_b_05[NUMSAMPLE_05]=
    {
        0x20008000,     //
        0x10000000,     //
        0x00001000,
        0x80000000,
        0x80000000,
        0x00008000,
        0x00004000,
        0x00002000,
        0x40000000,
        0x20000000,
        0x10000000,
        0x00000002,     // H*L test 1
        0x00000002,     // H*L test 2
        0x00000002,     // H*L test 3
        0x10000000,
    };


fract32 desired_05[NUMSAMPLE_05] =
    {
        0x10006001,         // Simple addition
        0xF0000000,         //
        0xFFFFF000,         //
        0xF0000000,         //
        0xFFFFF000,         //
        0x00000001,         // LSB
        0x00000000,         // round up
        0x00000000,         // truncate to 0
        0x00000001,         // LSB
        0x00000001,         // round up
        0x00000000,         // truncate to 0
        0x00000001,         // LSB
        0x00000001,         // round up
        0x00000000,         // truncate to 0
        0x00000000,         // must zero
    };

void test_05_fr32_vector_mul()
{
    fract32 output[NUMSAMPLE_05];
    int i;


        // clear output buffer
    clearBuffer( output, NUMSAMPLE_05);
        // test subtraction. Sample is less than NUMSAMPLE_05 to test the count parameter
    fr32_vector_mul( buf_a_05, buf_b_05, output, NUMSAMPLE_05-1);

    for ( i=0; i<NUMSAMPLE_05; i++)
    {
        if ( output[i] != desired_05[i] )
        {
            printf( "test_05 NG :output[%2d] = 0x%08X but should be 0x%08X\n", i, output[i], desired_05[i] );
            return;
        }
    }
    printf ("test_05 OK\n");
}

#undef TAPS_05
#undef NUMSAMPLE_05


/*
 * Basic test to see scalar + vector addition.
 */
#define NUMSAMPLE_06 15

fract32 buf_a_06[NUMSAMPLE_06]=
    {
        0x40008000,     // summation test for 4 partial products
        0x80000000,     // -1*H,
        0x80000000,     // -1*L
        0x10000000,     // H*-1
        0x00001000,     // L*-1
        0x00008000,     // L*L test 1
        0x00008000,     // L*L test 2
        0x00008000,     // L*L test 3
        0x00000002,     // H*L test 1
        0x00000002,     // H*L test 2
        0x00000002,     // H*L test 3
        0x40000000,
        0x20000000,
        0x10000000,
        0x11110000,     // dummy
    };


fract32 buf_b_06[NUMSAMPLE_06]=
    {
        0x20008000,     //
        0x10000000,     //
        0x00001000,
        0x80000000,
        0x80000000,
        0x00008000,
        0x00004000,
        0x00002000,
        0x40000000,
        0x20000000,
        0x10000000,
        0x00000002,     // H*L test 1
        0x00000002,     // H*L test 2
        0x00000002,     // H*L test 3
        0x10000000,
    };


fract32 desired_06[NUMSAMPLE_06] =
    {
        0x10006001,         // Simple addition
        0xF0000000,         //
        0xFFFFF000,         //
        0xF0000000,         //
        0xFFFFF000,         //
        0x00000001,         // LSB
        0x00000000,         // round up
        0x00000000,         // truncate to 0
        0x00000001,         // LSB
        0x00000001,         // round up
        0x00000000,         // truncate to 0
        0x00000001,         // LSB
        0x00000001,         // round up
        0x00000000,         // truncate to 0
        0x00000000,         // must zero
    };

void test_06_fr32_vector_mul_svv()
{
    fract32 output[NUMSAMPLE_06];
    int i;


        // clear output buffer
    clearBuffer( output, NUMSAMPLE_06);
        // test subtraction. Sample is less than NUMSAMPLE_06 to test the count parameter
    for( i=0; i<NUMSAMPLE_06-1; i++)
        fr32_vector_mul_svv( buf_a_06[i], &buf_b_06[i], &output[i], 1);

    for ( i=0; i<NUMSAMPLE_06; i++)
    {
        if ( output[i] != desired_06[i] )
        {
            printf( "test_06 NG :output[%2d] = 0x%08X but should be 0x%08X\n", i, output[i], desired_06[i] );
            return;
        }
    }
    printf ("test_06 OK\n");
}

#undef TAPS_06
#undef NUMSAMPLE_06



/*
 * Basic test to see scalar + vector addition.
 */
#define NUMSAMPLE_07 61

fract32 buf_ar_07[NUMSAMPLE_07]=
    {
        0x00000000,     // offset check,

        0x40008000,     // summation test for 4 partial products
        0x80000000,     // -1*H,
        0x80000000,     // -1*L
        0x10000000,     // H*-1
        0x00001000,     // L*-1
        0x00008000,     // L*L test 1
        0x00008000,     // L*L test 2
        0x00008000,     // L*L test 3
        0x00000002,     // H*L test 1
        0x00000002,     // H*L test 2
        0x00000002,     // H*L test 3
        0x40000000,
        0x20000000,
        0x10000000,
        0x00000000,     // dummy

        0x40008000,     // summation test for 4 partial products
        0x80000000,     // -1*H,
        0x80000000,     // -1*L
        0x10000000,     // H*-1
        0x00001000,     // L*-1
        0x00008000,     // L*L test 1
        0x00008000,     // L*L test 2
        0x00008000,     // L*L test 3
        0x00000002,     // H*L test 1
        0x00000002,     // H*L test 2
        0x00000002,     // H*L test 3
        0x40000000,
        0x20000000,
        0x10000000,
        0x00000000,     // dummy

        0x00000000,
        0x00000000,
        0x00000000,
        0x00000000,
        0x00000000,
        0x00000000,
        0x00000000,
        0x00000000,
        0x00000000,
        0x00000000,
        0x00000000,
        0x00000000,
        0x00000000,
        0x00000000,
        0x00000000,

        0x00000000,
        0x00000000,
        0x00000000,
        0x00000000,
        0x00000000,
        0x00000000,
        0x00000000,
        0x00000000,
        0x00000000,
        0x00000000,
        0x00000000,
        0x00000000,
        0x00000000,
        0x00000000,
        0x00000000,


    };


fract32 buf_ai_07[NUMSAMPLE_07]=
    {
        0x00000000,

        0x00000000,
        0x00000000,
        0x00000000,
        0x00000000,
        0x00000000,
        0x00000000,
        0x00000000,
        0x00000000,
        0x00000000,
        0x00000000,
        0x00000000,
        0x00000000,
        0x00000000,
        0x00000000,
        0x00000000,

        0x00000000,
        0x00000000,
        0x00000000,
        0x00000000,
        0x00000000,
        0x00000000,
        0x00000000,
        0x00000000,
        0x00000000,
        0x00000000,
        0x00000000,
        0x00000000,
        0x00000000,
        0x00000000,
        0x00000000,

        0x40008000,     // summation test for 4 partial products
        0x80000000,     // -1*H,
        0x80000000,     // -1*L
        0x10000000,     // H*-1
        0x00001000,     // L*-1
        0x00008000,     // L*L test 1
        0x00008000,     // L*L test 2
        0x00008000,     // L*L test 3
        0x00000002,     // H*L test 1
        0x00000002,     // H*L test 2
        0x00000002,     // H*L test 3
        0x40000000,
        0x20000000,
        0x10000000,
        0x00000000,     // dummy

        0x40008000,     // summation test for 4 partial products
        0x80000000,     // -1*H,
        0x80000000,     // -1*L
        0x10000000,     // H*-1
        0x00001000,     // L*-1
        0x00008000,     // L*L test 1
        0x00008000,     // L*L test 2
        0x00008000,     // L*L test 3
        0x00000002,     // H*L test 1
        0x00000002,     // H*L test 2
        0x00000002,     // H*L test 3
        0x40000000,
        0x20000000,
        0x10000000,
        0x00000000,     // dummy

    };


fract32 buf_br_07[NUMSAMPLE_07]=
    {
        0x00000000,

        0x20008000,     //
        0x10000000,     //
        0x00001000,
        0x80000000,
        0x80000000,
        0x00008000,
        0x00004000,
        0x00002000,
        0x40000000,
        0x20000000,
        0x10000000,
        0x00000002,     // H*L test 1
        0x00000002,     // H*L test 2
        0x00000002,     // H*L test 3
        0x00000000,

        0x00000000,
        0x00000000,
        0x00000000,
        0x00000000,
        0x00000000,
        0x00000000,
        0x00000000,
        0x00000000,
        0x00000000,
        0x00000000,
        0x00000000,
        0x00000000,
        0x00000000,
        0x00000000,
        0x00000000,

        0x20008000,     //
        0x10000000,     //
        0x00001000,
        0x80000000,
        0x80000000,
        0x00008000,
        0x00004000,
        0x00002000,
        0x40000000,
        0x20000000,
        0x10000000,
        0x00000002,     // H*L test 1
        0x00000002,     // H*L test 2
        0x00000002,     // H*L test 3
        0x00000000,

        0x00000000,
        0x00000000,
        0x00000000,
        0x00000000,
        0x00000000,
        0x00000000,
        0x00000000,
        0x00000000,
        0x00000000,
        0x00000000,
        0x00000000,
        0x00000000,
        0x00000000,
        0x00000000,
        0x00000000,


    };

fract32 buf_bi_07[NUMSAMPLE_07]=
    {
        0x00000000,

        0x00000000,
        0x00000000,
        0x00000000,
        0x00000000,
        0x00000000,
        0x00000000,
        0x00000000,
        0x00000000,
        0x00000000,
        0x00000000,
        0x00000000,
        0x00000000,
        0x00000000,
        0x00000000,
        0x00000000,

        0x20008000,     //
        0x10000000,     //
        0x00001000,
        0x80000000,
        0x80000000,
        0x00008000,
        0x00004000,
        0x00002000,
        0x40000000,
        0x20000000,
        0x10000000,
        0x00000002,     // H*L test 1
        0x00000002,     // H*L test 2
        0x00000002,     // H*L test 3
        0x00000000,

        0x00000000,
        0x00000000,
        0x00000000,
        0x00000000,
        0x00000000,
        0x00000000,
        0x00000000,
        0x00000000,
        0x00000000,
        0x00000000,
        0x00000000,
        0x00000000,
        0x00000000,
        0x00000000,
        0x00000000,

        0x20008000,     //
        0x10000000,     //
        0x00001000,
        0x80000000,
        0x80000000,
        0x00008000,
        0x00004000,
        0x00002000,
        0x40000000,
        0x20000000,
        0x10000000,
        0x00000002,     // H*L test 1
        0x00000002,     // H*L test 2
        0x00000002,     // H*L test 3
        0x00000000,


    };

fract32 desired_r_07[NUMSAMPLE_07] ;

fract32 desired_i_07[NUMSAMPLE_07] ;

void test_07_fr32_vector_complex_mul()
{
    fract32 output_r[NUMSAMPLE_07], output_i[NUMSAMPLE_07];
    int i;


        // clear output buffer
    clearBuffer( output_r, NUMSAMPLE_07);
    clearBuffer( output_i, NUMSAMPLE_07);

    for ( i=0; i<NUMSAMPLE_07; i++)
    {
        desired_r_07[i] = mult_fr1x32x32( buf_ar_07[i], buf_br_07[i])
                        - mult_fr1x32x32( buf_ai_07[i], buf_bi_07[i]);
        desired_i_07[i] = mult_fr1x32x32( buf_ar_07[i], buf_bi_07[i])
                        + mult_fr1x32x32( buf_ai_07[i], buf_br_07[i]);
    }

    for ( i=46; i<NUMSAMPLE_07; i++)
    {
        desired_r_07[i] = mult_fr1x32x32( buf_ar_07[i], buf_br_07[i])
                        + neg_mult_fr1x32x32(buf_ai_07[i], buf_bi_07[i]);
    }

        // test subtraction. Sample is less than NUMSAMPLE_07 to test the count parameter
    fr32_vector_complex_mul( buf_ar_07, buf_ai_07, buf_br_07, buf_bi_07,
                                output_r, output_i, NUMSAMPLE_07);

    for ( i=0; i<NUMSAMPLE_07; i++)
    {
        if ( output_r[i] != desired_r_07[i] )
        {
            printf( "test_07 NG :output_r[%2d] = 0x%08X but should be 0x%08X\n", i, output_r[i], desired_r_07[i] );
            return;
        }
        if ( output_i[i] != desired_i_07[i] )
        {
            printf( "test_07 NG :output_i[%2d] = 0x%08X but should be 0x%08X\n", i, output_i[i], desired_i_07[i] );
            return;
        }
    }
    printf ("test_07 OK\n");
}

#undef TAPS_07
#undef NUMSAMPLE_07


/*
 * Basic test to see scalar + vector addition.
 */
#define NUMSAMPLE_08 5

fract32 buf_a_08[NUMSAMPLE_08]=
    {
        0x00000001,     //
        0xFFFFFFFF,     //
        0x80000001,
        0x80000000,
        0x20000000
    };


fract32 desired_08[NUMSAMPLE_08] =
    {
        0xFFFFFFFF,         // -1
        0x00000001,         // 1
        0x7FFFFFFF,
        0x7FFFFFFF,         // saturated
        0x00000000,         // 0 for count test
    };

void test_08_fr32_vector_neg()
{
    fract32 output[NUMSAMPLE_08];
    int i;


        // clear output buffer
    clearBuffer( output, NUMSAMPLE_08);
        // test subtraction. Sample is less than NUMSAMPLE_08 to test the count parameter
    fr32_vector_neg( buf_a_08, output, NUMSAMPLE_08-1);

    for ( i=0; i<NUMSAMPLE_08; i++)
    {
        if ( output[i] != desired_08[i] )
        {
            printf( "test_08 NG :output[%2d] = 0x%08X but should be 0x%08X\n", i, output[i], desired_08[i] );
            return;
        }
    }
    printf ("test_08 OK\n");
}

#undef TAPS_08
#undef NUMSAMPLE_08

/*
 * Basic test to see scalar + vector addition.
 */
#define NUMSAMPLE_09 4



fract32 desired_09[NUMSAMPLE_09] =
    {
        0x3,
        0x3,
        0x3,
        0x00000000,         // 0 for count test
    };

void test_09_fr32_vector_fill()
{
    fract32 output[NUMSAMPLE_09];
    int i;


        // clear output buffer
    clearBuffer( output, NUMSAMPLE_09);
        // test subtraction. Sample is less than NUMSAMPLE_09 to test the count parameter
    fr32_vector_fill( 3, output, NUMSAMPLE_09-1);

    for ( i=0; i<NUMSAMPLE_09; i++)
    {
        if ( output[i] != desired_09[i] )
        {
            printf( "test_09 NG :output[%2d] = 0x%08X but should be 0x%08X\n", i, output[i], desired_09[i] );
            return;
        }
    }
    printf ("test_09 OK\n");
}

#undef TAPS_09
#undef NUMSAMPLE_09

